#ifndef WIRE_CPP
#define WIRE_CPP

///////////////////////////////////////////////////////////////////////////////
//
// $Header: $
//
// NAME: Wire
//
// BASE CLASSES: DataPathComponent.
//
// PURPOSE: This class describes a simple Wire object.
//
// AUTHOR: Nicolas MOREAU & Brice DONVAL
//
//  COPYRIGHT NOTICE:
//
// The KOYOTE Project
// Copyright (C) 2003-2004 Nicolas MOREAU & Brice DONVAL
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  REVISION HISTORY:
// $Log: $
//
///////////////////////////////////////////////////////////////////////////////

//Includes

#include "Wire.hpp"

//Constants

const int WIRE_COORD_ACCURACY = 2;

//Externals

//Types and classes

//Member Functions

///////////////////////////////////////////////////////////////////////////////
//
// NAME: Wire
//
// DESCRIPTION: Constructor for Wire objects.
//
// PARAMETERS:
//
// RETURNS: N/A.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
Wire::Wire(Coordinate* pCoord,
	   const int nCoord,
	   const DisplayMode eDisplayMode,
	   const WireType eType,
	   const WireColor eColor,
	   const byte bEnd,
	   const byte bStart)
  : DataPathComponent(pCoord,nCoord,eDisplayMode)
{
  m_eType = eType;
  m_eColor = eColor;
  m_bStart = bStart;
  m_bEnd = bEnd;
  m_pwValue = new word[1];
  m_pwValue[0] = 0x00000000;
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: ~Wire
//
// DESCRIPTION: Destructor for Wire objects.
//
// PARAMETERS: None.
//
// RETURNS: N/A.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
Wire::~Wire(void)
{
  delete [] m_pwValue;
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: GetValue
//
// DESCRIPTION: This public method is used to get a value from the object.
//
// PARAMETERS:
//  - nIndex : Index of the value we want to get.
//  - eAskMode : The get mode. Legal Values are :
//           * ASK_PASV (0) : Passive mode (m_eIsActivated unchanged)
//           * ASK_ACTV (1) : Active mode (m_eIsActivated = true)
//
// RETURNS: The needed value or 0x00000000 if the value doesn't exist.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
word
Wire::GetValue(const int nIndex,const AskMode eAskMode)
{
  switch(nIndex)
    {
    case 0:
    case 1:
      switch(eAskMode)
	{
	case ASK_ACTV:
	  m_eIsActivated = true;
	  break;
	case ASK_PASV:	  
	default:
	  break;
	}
      //      if(m_eIsUpToDate == false)
	{
	  m_pwValue[0] = 0x00000000;
	  word wPrev;
	  for(int nPrev = 0 ; nPrev < m_nInput ; nPrev++)
	    {
	      wPrev = m_ppInput[nPrev]->GetValue(m_pnOutput[nPrev],eAskMode);
	      m_pwValue[0] |= wPrev;
	    }
	  m_pwValue[0] = Core::Sel(m_pwValue[0],m_bEnd,m_bStart);
	  // 	  m_eIsUpToDate = true;
	}
      break;
    default:
      return 0x00000000;
    }
  if(nIndex == 1)
    return m_pwValue[0] << m_bStart;
  return m_pwValue[0];
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: Display
//
// DESCRIPTION: This method is used to display the component on the screen.
//
// PARAMETERS: None.
//
// RETURNS: None.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
void
Wire::Display(void)
{
  switch(m_eDisplayMode)
    {
    default:
    case DISPLAY_GRAPHIC:
      
      GdkPoint Arrow[3];
      int x0;
      int x1;
      int y0;
      int y1;
      WireColor eColor;
      
      ColorMap = gdk_window_get_colormap(pDrawingArea->window);
      gc = gdk_gc_new(pDrawingArea->window);
      
      for(int nCoord = 0 ; nCoord < (m_nCoord - 1) ; nCoord++)
	{
	  x0 = m_pCoord[nCoord].X;
	  x1 = m_pCoord[nCoord+1].X;
	  y0 = m_pCoord[nCoord].Y;
	  y1 = m_pCoord[nCoord+1].Y;
	  eColor = (m_eIsActivated == true)? WIRE_BLUE : m_eColor;
	  switch (eColor)
	    {
	    default:
	    case WIRE_BLACK:
	      Color.red = 0;
	      Color.green = 0;
	      Color.blue = 0;
	      break;
	      
	    case WIRE_BLUE:
	      Color.red = 0;
	      Color.green = 0;
	      Color.blue = 50000;
	      break;
	      
	    case WIRE_GREEN:
	      Color.red = 0;
	      Color.green = 40000;
	      Color.blue = 40000;
	      break;
	    }
	  
	  gdk_color_alloc(ColorMap,&Color);
	  gdk_gc_set_foreground(gc,&Color);
	  
	  if((m_bEnd - m_bStart) > 0) // Bold
	    {
	      if(m_eType == WIRE_ARROW)
		{
		  if(nCoord == (m_nCoord - 2))
		    {
		      if(x0 == x1)
			{
			  if(y0 < y1)
			    {
			      gdk_draw_line(pPixmap,gc,x0,y0,x1,y1-3);
			      gdk_draw_line(pPixmap,gc,x0-1,y0,x1-1,y1-3);
			      gdk_draw_line(pPixmap,gc,x0+1,y0,x1+1,y1-3);
			    }
			  else if(y0>y1)
			    {
			      gdk_draw_line(pPixmap,gc,x0,y0,x1,y1+3);
			      gdk_draw_line(pPixmap,gc,x0-1,y0,x1-1,y1+3);
			      gdk_draw_line(pPixmap,gc,x0+1,y0,x1+1,y1+3);
			    }
			}
		      else if(y0 == y1)
			{
			  if(x0 < x1)
			    {
			      gdk_draw_line(pPixmap,gc,x0,y0,x1-3,y1);
			      gdk_draw_line(pPixmap,gc,x0,y0-1,x1-3,y1-1);
			      gdk_draw_line(pPixmap,gc,x0,y0+1,x1-3,y1+1);
			    }
			  else if(x0 > x1)
			    {
			      gdk_draw_line(pPixmap,gc,x0,y0,x1+3,y1);
			      gdk_draw_line(pPixmap,gc,x0,y0-1,x1+3,y1-1);
			      gdk_draw_line(pPixmap,gc,x0,y0+1,x1+3,y1+1);
			    }
			}
		    }
		  else
		    {
		      if(x0 == x1)
			{
			 gdk_draw_line(pPixmap,gc,x0,y0,x1,y1);
			 gdk_draw_line(pPixmap,gc,x0-1,y0,x1-1,y1);
			 gdk_draw_line(pPixmap,gc,x0+1,y0,x1+1,y1);
			}
		      else if(y0 == y1)
			{
			  gdk_draw_line(pPixmap,gc,x0,y0,x1,y1);
			  gdk_draw_line(pPixmap,gc,x0,y0-1,x1,y1-1);
			  gdk_draw_line(pPixmap,gc,x0,y0+1,x1,y1+1);
			}
		    }
		}
	      else if(m_eType == WIRE_CIRCLE)
		{
		  if(x0 == x1)
		    {
		      gdk_draw_line(pPixmap,gc,x0,y0,x1,y1);
		      gdk_draw_line(pPixmap,gc,x0-1,y0,x1-1,y1);
		      gdk_draw_line(pPixmap,gc,x0+1,y0,x1+1,y1);
		    }
		  else if(y0 == y1)
		    {
		      gdk_draw_line(pPixmap,gc,x0,y0,x1,y1);
		      gdk_draw_line(pPixmap,gc,x0,y0-1,x1,y1-1);
		      gdk_draw_line(pPixmap,gc,x0,y0+1,x1,y1+1);
		    }
		}
	    }
	  else if((m_bEnd - m_bStart) == 0) // Thick
	    {
	      gdk_draw_line (pPixmap,gc,x0,y0,x1,y1);
	    }
	  if(m_eType == WIRE_CIRCLE)
	    {
	      if(nCoord == (m_nCoord - 2))
		{
		  if((m_bEnd - m_bStart) > 0) // Bold
		    gdk_draw_arc(pPixmap,gc,TRUE,x1-4,y1-4,8,8,0,360*64);
		  else // Thick
		    gdk_draw_arc(pPixmap,gc,TRUE,x1-3,y1-4,7,7,0,360*64);
		}
	    }
	  else if(m_eType == WIRE_ARROW)
	    {
	      if(x0 < x1)
		{
		  Arrow[0].x = x1+1; Arrow[0].y = y1;
		  Arrow[1].x = x1-5; Arrow[1].y = y1-5;
		  Arrow[2].x = x1-5; Arrow[2].y = y1+5;
		}
	      else if (x0 > x1)
		{
		  Arrow[0].x = x1-1; Arrow[0].y = y1;
		  Arrow[1].x = x1+5; Arrow[1].y = y1-5;
		  Arrow[2].x = x1+5; Arrow[2].y = y1+5;
		}
	      else if (y0 < y1)
		{
		  Arrow[0].x = x1;   Arrow[0].y = y1+1;
		  Arrow[1].x = x1-5; Arrow[1].y = y1-5;
		  Arrow[2].x = x1+5; Arrow[2].y = y1-5;
		}
	      else if (y0 > y1)
		{
		  Arrow[0].x = x1;   Arrow[0].y = y1-1;
		  Arrow[1].x = x1-5; Arrow[1].y = y1+5;
		  Arrow[2].x = x1+5; Arrow[2].y = y1+5;
		}
	      if(nCoord == (m_nCoord - 2))
		gdk_draw_polygon(pPixmap,gc,TRUE,Arrow,3);
	    }
	}
      gdk_gc_destroy(gc);
      gtk_widget_queue_draw_area (pDrawingArea,
				  0,0,
				  pDrawingArea->allocation.width,
				  pDrawingArea->allocation.height);

      break;
    case DISPLAY_TEXT:

      break;
    }
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: MouseIsOver
//
// DESCRIPTION: This public method decides whether mouse is over the object.
//
// PARAMETERS: Cursor Position (Coordinate).
//
// RETURNS: A boolean:
//  - false if the Mouse is outside the component area.
//  - true if the Mouse is inside the component area.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
bool
Wire::MouseIsOver(Coordinate* pCursor)
{
  bool eIsOver = false;
  int nCoord;
  for(nCoord = 0 ; (nCoord < m_nCoord-1) && (eIsOver == false) ; nCoord++)
    {
      if(( pCursor->X >= (m_pCoord[ nCoord ].X - WIRE_COORD_ACCURACY) ) &&
	 ( pCursor->X <= (m_pCoord[nCoord+1].X + WIRE_COORD_ACCURACY) ) &&
	 ( pCursor->Y >= (m_pCoord[ nCoord ].Y - WIRE_COORD_ACCURACY) ) &&
	 ( pCursor->Y <= (m_pCoord[nCoord+1].Y + WIRE_COORD_ACCURACY) ))
	eIsOver = true;
    }
  return eIsOver; 
}

#endif // WIRE_CPP
