#ifndef MONOCYCLECONTROL_CPP
#define MONOCYCLECONTROL_CPP

///////////////////////////////////////////////////////////////////////////////
//
// $Header: $
//
// NAME: MonocycleControl
//
// BASE CLASSES: None.
//
// PURPOSE: A monocycle datapath control unit implementation using a PLA.
//
// AUTHOR: Nicolas MOREAU
//
//  COPYRIGHT NOTICE:
//
// The KOYOTE Project
// Copyright (C) 2003-2004 Nicolas MOREAU & Brice DONVAL
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  REVISION HISTORY:
// $Log: $
//
///////////////////////////////////////////////////////////////////////////////

//Includes

#include "MonocycleControl.hpp"

//Constants

//This array is used to match a number with the corresponding monocycle control line's name
const char *szMonocycleControlLine[MAX_MONOCYCLE_CONTROL_LINES]={
  "RegDst  " , "Branch  " , "MemRead " , "MemToReg" ,
  "ALUOp   " , "MemWrite" , "ReadSrc " , "RegWrite" ,
  "Jump    " , "JumpSrc " , "Link    " , "Unsigned" ,
  "DataSrc " , "ZeroCmp " , "ZeroInv " , "ShiftOp " };

//Externals

//Types and classes

//Member Functions

///////////////////////////////////////////////////////////////////////////////
//
// NAME: Result
//
// DESCRIPTION: Public method used to get a control line value.
//
// PARAMETERS:
//  - nLine : The line we want to get.
//  - wInstruction : The instruction to decode.
//
// RETURNS: The needed control line or 0x00 if line doesn't exist.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::Result(int nLine,word wInstruction)
{
  byte r;
  switch(nLine)
    {
    case _RegDst_:
      r = RegDst(wInstruction);
      break;
    case _Branch_:
      r = Branch(wInstruction);
      break;
    case _MemRead_:
      r = MemRead(wInstruction);
      break;
    case _MemToReg_:
      r = MemToReg(wInstruction); 
      break;
    case _ALUOp_:
      r = ALUOp(wInstruction);
      break;
    case _MemWrite_:
      r = MemWrite(wInstruction);
      break;
    case _ReadSrc_:
      r = ReadSrc(wInstruction);
      break;
    case _RegWrite_:
      r = RegWrite(wInstruction);
      break;
    case _Jump_:
      r = Jump(wInstruction);
      break;
    case _JumpSrc_:
      r = JumpSrc(wInstruction);
      break;
    case _Link_:
      r = Link(wInstruction);
      break;
    case _Unsigned_:
      r = Unsigned(wInstruction);
      break;
    case _DataSrc_:
      r = DataSrc(wInstruction);
      break;
    case _ZeroCmp_:
      r = ZeroCmp(wInstruction);
      break;
    case _ZeroInv_:
      r = ZeroInv(wInstruction);
      break;
    case _ShiftOp_:
      r = ShiftOp(wInstruction);
      break;
    default:
      return 0x00;
    }
  return r;
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: is_sll
//
// DESCRIPTION: Private method used to know whether instruction is a SLL.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: TRUE (1) or FALSE (0).
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::is_sll(word wInstruction)
{
  return Core::Bit(

		   ~Core::Bit(wInstruction,31)&
		   ~Core::Bit(wInstruction,30)&
		   ~Core::Bit(wInstruction,29)&
		   ~Core::Bit(wInstruction,28)&
		   ~Core::Bit(wInstruction,27)&
		   ~Core::Bit(wInstruction,26)&
		   ~Core::Bit(wInstruction,5)&
		   ~Core::Bit(wInstruction,4)&
		   ~Core::Bit(wInstruction,3)&
		   ~Core::Bit(wInstruction,2)&
		   ~Core::Bit(wInstruction,1)&
		   ~Core::Bit(wInstruction,0)

		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: is_srl
//
// DESCRIPTION: Private method used to know whether instruction is a SRL.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: TRUE (1) or FALSE (0).
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::is_srl(word wInstruction)
{
  return Core::Bit(

		   ~Core::Bit(wInstruction,31)&
		   ~Core::Bit(wInstruction,30)&
		   ~Core::Bit(wInstruction,29)&
		   ~Core::Bit(wInstruction,28)&
		   ~Core::Bit(wInstruction,27)&
		   ~Core::Bit(wInstruction,26)&
		   ~Core::Bit(wInstruction,5)&
		   ~Core::Bit(wInstruction,4)&
		   ~Core::Bit(wInstruction,3)&
		   ~Core::Bit(wInstruction,2)&
		   Core::Bit(wInstruction,1)&
		   ~Core::Bit(wInstruction,0)

		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: is_sra
//
// DESCRIPTION: Private method used to know whether instruction is a SRA.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: TRUE (1) or FALSE (0).
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::is_sra(word wInstruction)
{
  return Core::Bit(

		   ~Core::Bit(wInstruction,31)&
		   ~Core::Bit(wInstruction,30)&
		   ~Core::Bit(wInstruction,29)&
		   ~Core::Bit(wInstruction,28)&
		   ~Core::Bit(wInstruction,27)&
		   ~Core::Bit(wInstruction,26)&
		   ~Core::Bit(wInstruction,5)&
		   ~Core::Bit(wInstruction,4)&
		   ~Core::Bit(wInstruction,3)&
		   ~Core::Bit(wInstruction,2)&
		   Core::Bit(wInstruction,1)&
		   Core::Bit(wInstruction,0)

		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: is_sllv
//
// DESCRIPTION: Private method used to know whether instruction is a SLLV.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: TRUE (1) or FALSE (0).
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::is_sllv(word wInstruction)
{
  return Core::Bit(

		   ~Core::Bit(wInstruction,31)&
		   ~Core::Bit(wInstruction,30)&
		   ~Core::Bit(wInstruction,29)&
		   ~Core::Bit(wInstruction,28)&
		   ~Core::Bit(wInstruction,27)&
		   ~Core::Bit(wInstruction,26)&
		   ~Core::Bit(wInstruction,5)&
		   ~Core::Bit(wInstruction,4)&
		   ~Core::Bit(wInstruction,3)&
		   Core::Bit(wInstruction,2)&
		   ~Core::Bit(wInstruction,1)&
		   ~Core::Bit(wInstruction,0)

		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: is_srlv
//
// DESCRIPTION: Private method used to know whether instruction is a SRLV.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: TRUE (1) or FALSE (0).
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::is_srlv(word wInstruction)
{
  return Core::Bit(

		   ~Core::Bit(wInstruction,31)&
		   ~Core::Bit(wInstruction,30)&
		   ~Core::Bit(wInstruction,29)&
		   ~Core::Bit(wInstruction,28)&
		   ~Core::Bit(wInstruction,27)&
		   ~Core::Bit(wInstruction,26)&
		   ~Core::Bit(wInstruction,5)&
		   ~Core::Bit(wInstruction,4)&
		   ~Core::Bit(wInstruction,3)&
		   Core::Bit(wInstruction,2)&
		   Core::Bit(wInstruction,1)&
		   ~Core::Bit(wInstruction,0)

		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: is_srav
//
// DESCRIPTION: Private method used to know whether instruction is a SRAV.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: TRUE (1) or FALSE (0).
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::is_srav(word wInstruction)
{
  return Core::Bit(

		   ~Core::Bit(wInstruction,31)&
		   ~Core::Bit(wInstruction,30)&
		   ~Core::Bit(wInstruction,29)&
		   ~Core::Bit(wInstruction,28)&
		   ~Core::Bit(wInstruction,27)&
		   ~Core::Bit(wInstruction,26)&
		   ~Core::Bit(wInstruction,5)&
		   ~Core::Bit(wInstruction,4)&
		   ~Core::Bit(wInstruction,3)&
		   Core::Bit(wInstruction,2)&
		   Core::Bit(wInstruction,1)&
		   Core::Bit(wInstruction,0)

		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: is_jr
//
// DESCRIPTION: Private method used to know whether instruction is a JR.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: TRUE (1) or FALSE (0).
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::is_jr(word wInstruction)
{
  return Core::Bit(

		   ~Core::Bit(wInstruction,31)&
		   ~Core::Bit(wInstruction,30)&
		   ~Core::Bit(wInstruction,29)&
		   ~Core::Bit(wInstruction,28)&
		   ~Core::Bit(wInstruction,27)&
		   ~Core::Bit(wInstruction,26)&
		   ~Core::Bit(wInstruction,5)&
		   ~Core::Bit(wInstruction,4)&
		   Core::Bit(wInstruction,3)&
		   ~Core::Bit(wInstruction,2)&
		   ~Core::Bit(wInstruction,1)&
		   ~Core::Bit(wInstruction,0)
		   
		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: is_jalr
//
// DESCRIPTION: Private method used to know whether instruction is a JALR.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: TRUE (1) or FALSE (0).
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::is_jalr(word wInstruction)
{
  return Core::Bit(
		   
		   ~Core::Bit(wInstruction,31)&
		   ~Core::Bit(wInstruction,30)&
		   ~Core::Bit(wInstruction,29)&
		   ~Core::Bit(wInstruction,28)&
		   ~Core::Bit(wInstruction,27)&
		   ~Core::Bit(wInstruction,26)&
		   ~Core::Bit(wInstruction,5)&
		   ~Core::Bit(wInstruction,4)&
		   Core::Bit(wInstruction,3)&
		   ~Core::Bit(wInstruction,2)&
		   ~Core::Bit(wInstruction,1)&
		   Core::Bit(wInstruction,0)
		   
		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: is_add
//
// DESCRIPTION: Private method used to know whether instruction is a ADD.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: TRUE (1) or FALSE (0).
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::is_add(word wInstruction)
{
  return Core::Bit(
		   
		   ~Core::Bit(wInstruction,31)&
		   ~Core::Bit(wInstruction,30)&
		   ~Core::Bit(wInstruction,29)&
		   ~Core::Bit(wInstruction,28)&
		   ~Core::Bit(wInstruction,27)&
		   ~Core::Bit(wInstruction,26)&
		   Core::Bit(wInstruction,5)&
		   ~Core::Bit(wInstruction,4)&
		   ~Core::Bit(wInstruction,3)&
		   ~Core::Bit(wInstruction,2)&
		   ~Core::Bit(wInstruction,1)&
		   ~Core::Bit(wInstruction,0)
		   
		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: is_addu
//
// DESCRIPTION: Private method used to know whether instruction is a ADDU.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: TRUE (1) or FALSE (0).
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::is_addu(word wInstruction)
{
  return Core::Bit(
		   
		   ~Core::Bit(wInstruction,31)&
		   ~Core::Bit(wInstruction,30)&
		   ~Core::Bit(wInstruction,29)&
		   ~Core::Bit(wInstruction,28)&
		   ~Core::Bit(wInstruction,27)&
		   ~Core::Bit(wInstruction,26)&
		   Core::Bit(wInstruction,5)&
		   ~Core::Bit(wInstruction,4)&
		   ~Core::Bit(wInstruction,3)&
		   ~Core::Bit(wInstruction,2)&
		   ~Core::Bit(wInstruction,1)&
		   Core::Bit(wInstruction,0)
		   
		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: is_sub
//
// DESCRIPTION: Private method used to know whether instruction is a SUB.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: TRUE (1) or FALSE (0).
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::is_sub(word wInstruction)
{
  return Core::Bit(
		   
		   ~Core::Bit(wInstruction,31)&
		   ~Core::Bit(wInstruction,30)&
		   ~Core::Bit(wInstruction,29)&
		   ~Core::Bit(wInstruction,28)&
		   ~Core::Bit(wInstruction,27)&
		   ~Core::Bit(wInstruction,26)&
		   Core::Bit(wInstruction,5)&
		   ~Core::Bit(wInstruction,4)&
		   ~Core::Bit(wInstruction,3)&
		   ~Core::Bit(wInstruction,2)&
		   Core::Bit(wInstruction,1)&
		   ~Core::Bit(wInstruction,0)
		   
		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: is_subu
//
// DESCRIPTION: Private method used to know whether instruction is a SUBU.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: TRUE (1) or FALSE (0).
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::is_subu(word wInstruction)
{
  return Core::Bit(
		   
		   ~Core::Bit(wInstruction,31)&
		   ~Core::Bit(wInstruction,30)&
		   ~Core::Bit(wInstruction,29)&
		   ~Core::Bit(wInstruction,28)&
		   ~Core::Bit(wInstruction,27)&
		   ~Core::Bit(wInstruction,26)&
		   Core::Bit(wInstruction,5)&
		   ~Core::Bit(wInstruction,4)&
		   ~Core::Bit(wInstruction,3)&
		   ~Core::Bit(wInstruction,2)&
		   Core::Bit(wInstruction,1)&
		   Core::Bit(wInstruction,0)
		   
		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: is_and
//
// DESCRIPTION: Private method used to know whether instruction is a AND.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: TRUE (1) or FALSE (0).
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::is_and(word wInstruction)
{
  return Core::Bit(
		   
		   ~Core::Bit(wInstruction,31)&
		   ~Core::Bit(wInstruction,30)&
		   ~Core::Bit(wInstruction,29)&
		   ~Core::Bit(wInstruction,28)&
		   ~Core::Bit(wInstruction,27)&
		   ~Core::Bit(wInstruction,26)&
		   Core::Bit(wInstruction,5)&
		   ~Core::Bit(wInstruction,4)&
		   ~Core::Bit(wInstruction,3)&
		   Core::Bit(wInstruction,2)&
		   ~Core::Bit(wInstruction,1)&
		   ~Core::Bit(wInstruction,0)
		   
		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: is_or
//
// DESCRIPTION: Private method used to know whether instruction is a OR.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: TRUE (1) or FALSE (0).
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::is_or(word wInstruction)
{
  return Core::Bit(
		   
		   ~Core::Bit(wInstruction,31)&
		   ~Core::Bit(wInstruction,30)&
		   ~Core::Bit(wInstruction,29)&
		   ~Core::Bit(wInstruction,28)&
		   ~Core::Bit(wInstruction,27)&
		   ~Core::Bit(wInstruction,26)&
		   Core::Bit(wInstruction,5)&
		   ~Core::Bit(wInstruction,4)&
		   ~Core::Bit(wInstruction,3)&
		   Core::Bit(wInstruction,2)&
		   ~Core::Bit(wInstruction,1)&
		   Core::Bit(wInstruction,0)
		   
		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: is_xor
//
// DESCRIPTION: Private method used to know whether instruction is a XOR.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: TRUE (1) or FALSE (0).
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::is_xor(word wInstruction)
{
  return Core::Bit(
		   
		   ~Core::Bit(wInstruction,31)&
		   ~Core::Bit(wInstruction,30)&
		   ~Core::Bit(wInstruction,29)&
		   ~Core::Bit(wInstruction,28)&
		   ~Core::Bit(wInstruction,27)&
		   ~Core::Bit(wInstruction,26)&
		   Core::Bit(wInstruction,5)&
		   ~Core::Bit(wInstruction,4)&
		   ~Core::Bit(wInstruction,3)&
		   Core::Bit(wInstruction,2)&
		   Core::Bit(wInstruction,1)&
		   ~Core::Bit(wInstruction,0)
		   
		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: is_nor
//
// DESCRIPTION: Private method used to know whether instruction is a NOR.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: TRUE (1) or FALSE (0).
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::is_nor(word wInstruction)
{
  return Core::Bit(
		   
		   ~Core::Bit(wInstruction,31)&
		   ~Core::Bit(wInstruction,30)&
		   ~Core::Bit(wInstruction,29)&
		   ~Core::Bit(wInstruction,28)&
		   ~Core::Bit(wInstruction,27)&
		   ~Core::Bit(wInstruction,26)&
		   Core::Bit(wInstruction,5)&
		   ~Core::Bit(wInstruction,4)&
		   ~Core::Bit(wInstruction,3)&
		   Core::Bit(wInstruction,2)&
		   Core::Bit(wInstruction,1)&
		   Core::Bit(wInstruction,0)
		   
		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: is_slt
//
// DESCRIPTION: Private method used to know whether instruction is a SLT.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: TRUE (1) or FALSE (0).
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::is_slt(word wInstruction)
{
  return Core::Bit(
		   
		   ~Core::Bit(wInstruction,31)&
		   ~Core::Bit(wInstruction,30)&
		   ~Core::Bit(wInstruction,29)&
		   ~Core::Bit(wInstruction,28)&
		   ~Core::Bit(wInstruction,27)&
		   ~Core::Bit(wInstruction,26)&
		   Core::Bit(wInstruction,5)&
		   ~Core::Bit(wInstruction,4)&
		   Core::Bit(wInstruction,3)&
		   ~Core::Bit(wInstruction,2)&
		   Core::Bit(wInstruction,1)&
		   ~Core::Bit(wInstruction,0)
		   
		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: is_sltu
//
// DESCRIPTION: Private method used to know whether instruction is a SLTU.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: TRUE (1) or FALSE (0).
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::is_sltu(word wInstruction)
{
  return Core::Bit(
		   
		   ~Core::Bit(wInstruction,31)&
		   ~Core::Bit(wInstruction,30)&
		   ~Core::Bit(wInstruction,29)&
		   ~Core::Bit(wInstruction,28)&
		   ~Core::Bit(wInstruction,27)&
		   ~Core::Bit(wInstruction,26)&
		   Core::Bit(wInstruction,5)&
		   ~Core::Bit(wInstruction,4)&
		   Core::Bit(wInstruction,3)&
		   ~Core::Bit(wInstruction,2)&
		   Core::Bit(wInstruction,1)&
		   Core::Bit(wInstruction,0)
		   
		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: is_bltz
//
// DESCRIPTION: Private method used to know whether instruction is a BLTZ.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: TRUE (1) or FALSE (0).
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::is_bltz(word wInstruction)
{
  return Core::Bit(
		   
		   ~Core::Bit(wInstruction,31)&
		   ~Core::Bit(wInstruction,30)&
		   ~Core::Bit(wInstruction,29)&
		   ~Core::Bit(wInstruction,28)&
		   ~Core::Bit(wInstruction,27)&
		   Core::Bit(wInstruction,26)&
		   ~Core::Bit(wInstruction,20)&
		   ~Core::Bit(wInstruction,19)&
		   ~Core::Bit(wInstruction,18)&
		   ~Core::Bit(wInstruction,17)&
		   ~Core::Bit(wInstruction,16)
		   
		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: is_bgez
//
// DESCRIPTION: Private method used to know whether instruction is a BGEZ.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: TRUE (1) or FALSE (0).
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::is_bgez(word wInstruction)
{
  return Core::Bit(
		   
		   ~Core::Bit(wInstruction,31)&
		   ~Core::Bit(wInstruction,30)&
		   ~Core::Bit(wInstruction,29)&
		   ~Core::Bit(wInstruction,28)&
		   ~Core::Bit(wInstruction,27)&
		   Core::Bit(wInstruction,26)&
		   ~Core::Bit(wInstruction,20)&
		   ~Core::Bit(wInstruction,19)&
		   ~Core::Bit(wInstruction,18)&
		   ~Core::Bit(wInstruction,17)&
		   Core::Bit(wInstruction,16)
		   
		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: is_bltzal
//
// DESCRIPTION: Private method used to know whether instruction is a BLTZAL.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: TRUE (1) or FALSE (0).
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::is_bltzal(word wInstruction)
{
  return Core::Bit(
		   
		   ~Core::Bit(wInstruction,31)&
		   ~Core::Bit(wInstruction,30)&
		   ~Core::Bit(wInstruction,29)&
		   ~Core::Bit(wInstruction,28)&
		   ~Core::Bit(wInstruction,27)&
		   Core::Bit(wInstruction,26)&
		   Core::Bit(wInstruction,20)&
		   ~Core::Bit(wInstruction,19)&
		   ~Core::Bit(wInstruction,18)&
		   ~Core::Bit(wInstruction,17)&
		   ~Core::Bit(wInstruction,16)
		   
		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: is_bgezal
//
// DESCRIPTION: Private method used to know whether instruction is a BGEZAL.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: TRUE (1) or FALSE (0).
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::is_bgezal(word wInstruction)
{
  return Core::Bit(
		   
		   ~Core::Bit(wInstruction,31)&
		   ~Core::Bit(wInstruction,30)&
		   ~Core::Bit(wInstruction,29)&
		   ~Core::Bit(wInstruction,28)&
		   ~Core::Bit(wInstruction,27)&
		   Core::Bit(wInstruction,26)&
		   Core::Bit(wInstruction,20)&
		   ~Core::Bit(wInstruction,19)&
		   ~Core::Bit(wInstruction,18)&
		   ~Core::Bit(wInstruction,17)&
		   Core::Bit(wInstruction,16)
		   
		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: is_j
//
// DESCRIPTION: Private method used to know whether instruction is a J.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: TRUE (1) or FALSE (0).
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::is_j(word wInstruction)
{
  return Core::Bit(
		   
		   ~Core::Bit(wInstruction,31)&
		   ~Core::Bit(wInstruction,30)&
		   ~Core::Bit(wInstruction,29)&
		   ~Core::Bit(wInstruction,28)&
		   Core::Bit(wInstruction,27)&
		   ~Core::Bit(wInstruction,26)
		   
		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: is_jal
//
// DESCRIPTION: Private method used to know whether instruction is a JAL.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: TRUE (1) or FALSE (0).
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::is_jal(word wInstruction)
{
  return Core::Bit(
		   
		   ~Core::Bit(wInstruction,31)&
		   ~Core::Bit(wInstruction,30)&
		   ~Core::Bit(wInstruction,29)&
		   ~Core::Bit(wInstruction,28)&
		   Core::Bit(wInstruction,27)&
		   Core::Bit(wInstruction,26)
		   
		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: is_beq
//
// DESCRIPTION: Private method used to know whether instruction is a BEQ.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: TRUE (1) or FALSE (0).
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::is_beq(word wInstruction)
{
  return Core::Bit(
		   
		   ~Core::Bit(wInstruction,31)&
		   ~Core::Bit(wInstruction,30)&
		   ~Core::Bit(wInstruction,29)&
		   Core::Bit(wInstruction,28)&
		   ~Core::Bit(wInstruction,27)&
		   ~Core::Bit(wInstruction,26)
		   
		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: is_bne
//
// DESCRIPTION: Private method used to know whether instruction is a BNE.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: TRUE (1) or FALSE (0).
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::is_bne(word wInstruction)
{
  return Core::Bit(
		   
		   ~Core::Bit(wInstruction,31)&
		   ~Core::Bit(wInstruction,30)&
		   ~Core::Bit(wInstruction,29)&
		   Core::Bit(wInstruction,28)&
		   ~Core::Bit(wInstruction,27)&
		   Core::Bit(wInstruction,26)
		   
		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: is_blez
//
// DESCRIPTION: Private method used to know whether instruction is a BLEZ.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: TRUE (1) or FALSE (0).
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::is_blez(word wInstruction)
{
  return Core::Bit(
		   
		   ~Core::Bit(wInstruction,31)&
		   ~Core::Bit(wInstruction,30)&
		   ~Core::Bit(wInstruction,29)&
		   Core::Bit(wInstruction,28)&
		   Core::Bit(wInstruction,27)&
		   ~Core::Bit(wInstruction,26)
		   
		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: is_bgtz
//
// DESCRIPTION: Private method used to know whether instruction is a BGTZ.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: TRUE (1) or FALSE (0).
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::is_bgtz(word wInstruction)
{
  return Core::Bit(
		   
		   ~Core::Bit(wInstruction,31)&
		   ~Core::Bit(wInstruction,30)&
		   ~Core::Bit(wInstruction,29)&
		   Core::Bit(wInstruction,28)&
		   Core::Bit(wInstruction,27)&
		   Core::Bit(wInstruction,26)
		   
		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: is_addi
//
// DESCRIPTION: Private method used to know whether instruction is a ADDI.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: TRUE (1) or FALSE (0).
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::is_addi(word wInstruction)
{
  return Core::Bit(
		   
		   ~Core::Bit(wInstruction,31)&
		   ~Core::Bit(wInstruction,30)&
		   Core::Bit(wInstruction,29)&
		   ~Core::Bit(wInstruction,28)&
		   ~Core::Bit(wInstruction,27)&
		   ~Core::Bit(wInstruction,26)
		   
		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: is_addiu
//
// DESCRIPTION: Private method used to know whether instruction is a ADDIU.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: TRUE (1) or FALSE (0).
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::is_addiu(word wInstruction)
{
  return Core::Bit(
		   
		   ~Core::Bit(wInstruction,31)&
		   ~Core::Bit(wInstruction,30)&
		   Core::Bit(wInstruction,29)&
		   ~Core::Bit(wInstruction,28)&
		   ~Core::Bit(wInstruction,27)&
		   Core::Bit(wInstruction,26)
		   
		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: is_slti
//
// DESCRIPTION: Private method used to know whether instruction is a SLTI.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: TRUE (1) or FALSE (0).
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::is_slti(word wInstruction)
{
  return Core::Bit(
		   
		   ~Core::Bit(wInstruction,31)&
		   ~Core::Bit(wInstruction,30)&
		   Core::Bit(wInstruction,29)&
		   ~Core::Bit(wInstruction,28)&
		   Core::Bit(wInstruction,27)&
		   ~Core::Bit(wInstruction,26)
		   
		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: is_sltiu
//
// DESCRIPTION: Private method used to know whether instruction is a SLTIU.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: TRUE (1) or FALSE (0).
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::is_sltiu(word wInstruction)
{
  return Core::Bit(
		   
		   ~Core::Bit(wInstruction,31)&
		   ~Core::Bit(wInstruction,30)&
		   Core::Bit(wInstruction,29)&
		   ~Core::Bit(wInstruction,28)&
		   Core::Bit(wInstruction,27)&
		   Core::Bit(wInstruction,26)
		   
		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: is_andi
//
// DESCRIPTION: Private method used to know whether instruction is a ANDI.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: TRUE (1) or FALSE (0).
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::is_andi(word wInstruction)
{
  return Core::Bit(
		   
		   ~Core::Bit(wInstruction,31)&
		   ~Core::Bit(wInstruction,30)&
		   Core::Bit(wInstruction,29)&
		   Core::Bit(wInstruction,28)&
		   ~Core::Bit(wInstruction,27)&
		   ~Core::Bit(wInstruction,26)
		   
		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: is_ori
//
// DESCRIPTION: Private method used to know whether instruction is a ORI.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: TRUE (1) or FALSE (0).
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::is_ori(word wInstruction)
{
  return Core::Bit(
		   
		   ~Core::Bit(wInstruction,31)&
		   ~Core::Bit(wInstruction,30)&
		   Core::Bit(wInstruction,29)&
		   Core::Bit(wInstruction,28)&
		   ~Core::Bit(wInstruction,27)&
		   Core::Bit(wInstruction,26)
		   
		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: is_xori
//
// DESCRIPTION: Private method used to know whether instruction is a XORI.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: TRUE (1) or FALSE (0).
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::is_xori(word wInstruction)
{
  return Core::Bit(
		   
		   ~Core::Bit(wInstruction,31)&
		   ~Core::Bit(wInstruction,30)&
		   Core::Bit(wInstruction,29)&
		   Core::Bit(wInstruction,28)&
		   Core::Bit(wInstruction,27)&
		   ~Core::Bit(wInstruction,26)
		   
		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: is_lw
//
// DESCRIPTION: Private method used to know whether instruction is a LW.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: TRUE (1) or FALSE (0).
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::is_lw(word wInstruction)
{
  return Core::Bit(
		   
		   Core::Bit(wInstruction,31)&
		   ~Core::Bit(wInstruction,30)&
		   ~Core::Bit(wInstruction,29)&
		   ~Core::Bit(wInstruction,28)&
		   Core::Bit(wInstruction,27)&
		   Core::Bit(wInstruction,26)
		   
		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: is_sw
//
// DESCRIPTION: Private method used to know whether instruction is a SW.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: TRUE (1) or FALSE (0).
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::is_sw(word wInstruction)
{
  return Core::Bit(
		   
		   Core::Bit(wInstruction,31)&
		   ~Core::Bit(wInstruction,30)&
		   Core::Bit(wInstruction,29)&
		   ~Core::Bit(wInstruction,28)&
		   Core::Bit(wInstruction,27)&
		   Core::Bit(wInstruction,26)
		   
		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: RegWrite
//
// DESCRIPTION: Private method used to get the REGWRITE control line.
//
// PARAMETERS:
//  - wInstruction : The iInstruction to decode.
//
// RETURNS: The REGWRITE control line.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::RegWrite(word wInstruction)
{
  return Core::Bit(
		   
		   is_sll(wInstruction)|
		   is_srl(wInstruction)|
		   is_sra(wInstruction)|
		   is_sllv(wInstruction)|
		   is_srlv(wInstruction)|
		   is_srav(wInstruction)|
		   is_add(wInstruction)|
		   is_addu(wInstruction)|
		   is_sub(wInstruction)|
		   is_subu(wInstruction)|
		   is_and(wInstruction)|
		   is_or(wInstruction)|
		   is_slt(wInstruction)|
		   is_sltu(wInstruction)|
		   is_addi(wInstruction)|
		   is_addiu(wInstruction)|
		   is_slti(wInstruction)|
		   is_sltiu(wInstruction)|
		   is_andi(wInstruction)|
		   is_ori(wInstruction)|
		   is_lw(wInstruction)|
		   is_jal(wInstruction)|
		   is_jalr(wInstruction)|
		   is_bltzal(wInstruction)|
		   is_bgezal(wInstruction)|
		   is_xor(wInstruction)|
		   is_xori(wInstruction)|
		   is_nor(wInstruction)
		   
		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: ReadSrc
//
// DESCRIPTION: Private method used to get the READSRC control line.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: The READSRC control line.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::ReadSrc(word wInstruction)
{
  return Core::Bit(
		   
		   is_addi(wInstruction)|
		   is_addiu(wInstruction)|
		   is_slti(wInstruction)|
		   is_sltiu(wInstruction)|
		   is_andi(wInstruction)|
		   is_ori(wInstruction)|
		   is_lw(wInstruction)|
		   is_sw(wInstruction)|
		   is_sll(wInstruction)|
		   is_srl(wInstruction)|
		   is_sra(wInstruction)|
		   is_xori(wInstruction)
		   
		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: MemWrite
//
// DESCRIPTION: Private method used to get the MEMWRITE control line.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: The MEMWRITE control line.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::MemWrite(word wInstruction)
{
  return Core::Bit(
		   
		   is_sw(wInstruction)
		   
		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: ALUOp
//
// DESCRIPTION: Private method used to get the ALUOP control line.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: The ALUOP control line.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::ALUOp(word wInstruction)
{
  return Core::Sel( 
		   ((is_sub(wInstruction)|
		     is_slt(wInstruction)|
		     is_subu(wInstruction)|
		     is_sltu(wInstruction)|
		     is_bltz(wInstruction)|
		     is_bgez(wInstruction)|
		     is_bltzal(wInstruction)|
		     is_bgezal(wInstruction)|
		     is_beq(wInstruction)|
		     is_bne(wInstruction)|
		     is_blez(wInstruction)|
		     is_bgtz(wInstruction)|
		     is_slti(wInstruction)|
		     is_sltiu(wInstruction))<<3) |
		   
		   ((is_xor(wInstruction)|
		     is_xori(wInstruction)|
		     is_nor(wInstruction)|
		     is_addu(wInstruction)|
		     is_subu(wInstruction)|
		     is_sltu(wInstruction))<<2) |
		   
		   ((is_bltz(wInstruction)|
		     is_bgez(wInstruction)|
		     is_bltzal(wInstruction)|
		     is_bgezal(wInstruction)|
		     is_beq(wInstruction)|
		     is_bne(wInstruction)|
		     is_blez(wInstruction)|
		     is_bgtz(wInstruction)|
		     is_sub(wInstruction)|
		     is_subu(wInstruction)|
		     is_slt(wInstruction)|
		     is_sltu(wInstruction)|
		     is_slti(wInstruction)|
		     is_sltiu(wInstruction)|
		     is_add(wInstruction)|
		     is_addu(wInstruction)|
		     is_addi(wInstruction)|
		     is_addiu(wInstruction)|
		     is_lw(wInstruction)|
		     is_sw(wInstruction))<<1) |
		   
		   (is_bltz(wInstruction)|
		    is_bgez(wInstruction)|
		    is_bltzal(wInstruction)|
		    is_bgezal(wInstruction)|
		    is_blez(wInstruction)|
		    is_bgtz(wInstruction)|
		    is_slt(wInstruction)|
		    is_sltu(wInstruction)|
		    is_or(wInstruction)|
		    is_ori(wInstruction)|
		    is_slti(wInstruction)|
		    is_sltiu(wInstruction)|
		    is_xor(wInstruction)|
		    is_xori(wInstruction) )

		   ,3,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: MemToReg
//
// DESCRIPTION: Private method used to get the MEMTOREG control line.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: The MEMTOREG control line.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::MemToReg(word wInstruction)
{
  return Core::Bit(
		   
		   is_lw(wInstruction)
		   
		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: MemRead
//
// DESCRIPTION: Private method used to get the MEMREAD control line.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: The MEMREAD control line.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::MemRead(word wInstruction)
{
  return Core::Bit(
		   
		   is_lw(wInstruction)
		   
		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: Branch
//
// DESCRIPTION: Private method used to get the BRANCH control line.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: The BRANCH control line.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::Branch(word wInstruction)
{
  return Core::Bit(
		   
		   is_beq(wInstruction)|
		   is_bne(wInstruction)|
		   is_blez(wInstruction)|
		   is_bgtz(wInstruction)|
		   is_bltz(wInstruction)|
		   is_bgez(wInstruction)|
		   is_bltzal(wInstruction)|
		   is_bgezal(wInstruction)
		   
		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: Jump
//
// DESCRIPTION: Private method used to get the JUMP control line.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: The JUMP control line.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::Jump(word wInstruction)
{
  return Core::Bit(
		   
		   is_jr(wInstruction)|
		   is_jalr(wInstruction)|
		   is_j(wInstruction)|
		   is_jal(wInstruction)
		   
		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: RegDst
//
// DESCRIPTION: Private method used to get the REGDST control line.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: The REGDST control line.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::RegDst(word wInstruction)
{
  return Core::Bit(
		   
		   is_sll(wInstruction)|
		   is_srl(wInstruction)|
		   is_sra(wInstruction)|
		   is_sllv(wInstruction)|
		   is_srlv(wInstruction)|
		   is_srav(wInstruction)|
		   is_add(wInstruction)|
		   is_addu(wInstruction)|
		   is_sub(wInstruction)|
		   is_subu(wInstruction)|
		   is_and(wInstruction)|
		   is_or(wInstruction)|
		   is_slt(wInstruction)|
		   is_sltu(wInstruction)|
		   is_xor(wInstruction)|
		   is_nor(wInstruction)
		   
		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: Link
//
// DESCRIPTION: Private method used to get the LINK control line.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: The LINK control line.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::Link(word wInstruction)
{
  return Core::Bit(
		   
		   is_jalr(wInstruction)|
		   is_jal(wInstruction)|
		   is_bltzal(wInstruction)|
		   is_bgezal(wInstruction)
		   
		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: ZeroInv
//
// DESCRIPTION: Private method used to get the ZEROINV control line.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: The ZEROINV control line.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::ZeroInv(word wInstruction)
{
  return Core::Bit(
		   
		   is_bne(wInstruction)|
		   is_bltz(wInstruction)|
		   is_blez(wInstruction)|
		   is_bltzal(wInstruction)
		   
		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: ZeroCmp
//
// DESCRIPTION: Private method used to get the ZEROCMP control line.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: The ZEROCMP control line.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::ZeroCmp(word wInstruction)
{
  return Core::Bit(
		   
		   is_blez(wInstruction)|
		   is_bgez(wInstruction)|
		   is_bgezal(wInstruction)
		   
		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: Unsigned
//
// DESCRIPTION: Private method used to get the UNSIGNED control line.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: The UNSIGNED control line.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::Unsigned(word wInstruction)
{
  return Core::Bit(
		   
		   is_addiu(wInstruction)|
		   is_sltiu(wInstruction)|
		   is_andi(wInstruction)|
		   is_ori(wInstruction)|
		   is_addu(wInstruction)|
		   is_sltu(wInstruction)|
		   is_subu(wInstruction)|
		   is_sll(wInstruction)|
		   is_srl(wInstruction)|
		   is_sra(wInstruction)|
		   is_xori(wInstruction)
		   
		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: DataSrc
//
// DESCRIPTION: Private method used to get the DATASRC control line.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: The DATASRC control line.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::DataSrc(word wInstruction)
{
  return Core::Bit(
		   
		   is_sll(wInstruction)|
		   is_srl(wInstruction)|
		   is_sra(wInstruction)|
		   is_sllv(wInstruction)|
		   is_srlv(wInstruction)|
		   is_srav(wInstruction)
		   
		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: JumpSrc
//
// DESCRIPTION: Private method used to get the JUMPSRC control line.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: The JUMPSRC control line.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::JumpSrc(word wInstruction)
{
  return Core::Bit(
		   
		   is_jr(wInstruction)|
		   is_jalr(wInstruction)
		   
		   ,0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: ShiftOp
//
// DESCRIPTION: Private method used to get the SHIFTOP control line.
//
// PARAMETERS:
//  - wInstruction : The instruction to decode.
//
// RETURNS: The SHIFTOP control line.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
MonocycleControl::ShiftOp(word wInstruction)
{
  return Core::Sel(
		   ((is_srav(wInstruction)|
		     is_srlv(wInstruction)|
		     is_sllv(wInstruction))<<2) |
		   ((is_sra(wInstruction)|
		     is_srav(wInstruction))<<1) |
		   (is_sll(wInstruction)|
		    is_sllv(wInstruction))
		   
		   ,2,0);
}

#endif // MONOCYCLECONTROL_CPP
